@@ -1,27 +1,26 @@
-import sys
-import re
 import logging
+import re
+import sys
 
-from jinja2 import Environment, FileSystemLoader
-
-from traitlets import (
-    HasTraits,
-    Unicode,
-    List,
-    Dict,
-    Bool,
-    default
-)
-from traitlets.config import Config
+from jinja2 import Environment
+from jinja2 import FileSystemLoader
+from jupyter_core.application import JupyterApp
+from jupyter_core.application import NoStart
 from tornado.log import LogFormatter
 from tornado.web import RedirectHandler
+from traitlets import Bool
+from traitlets import default
+from traitlets import Dict
+from traitlets import HasTraits
+from traitlets import List
+from traitlets import Unicode
+from traitlets.config import Config
 
-from jupyter_core.application import JupyterApp, NoStart
-
-from jupyter_server.serverapp import ServerApp
-from jupyter_server.transutils import _
-from jupyter_server.utils import url_path_join
 from .handler import ExtensionHandlerMixin
+from jupyter_server.serverapp import ServerApp
+from jupyter_server.transutils import _i18n
+from jupyter_server.utils import is_namespace_package
+from jupyter_server.utils import url_path_join
 
 # -----------------------------------------------------------------------------
 # Util functions and classes.
@@ -29,8 +28,7 @@ from .handler import ExtensionHandlerMixin
 
 
 def _preparse_for_subcommand(Application, argv):
-    """Preparse command line to look for subcommands.
-    """
+    """Preparse command line to look for subcommands."""
     # Read in arguments from command line.
     if len(argv) == 0:
         return
@@ -39,7 +37,7 @@ def _preparse_for_subcommand(Application, argv):
     if Application.subcommands and len(argv) > 0:
         # we have subcommands, and one may have been specified
         subc, subargv = argv[0], argv[1:]
-        if re.match(r'^\w(\-?\w)*$', subc) and subc in Application.subcommands:
+        if re.match(r"^\w(\-?\w)*$", subc) and subc in Application.subcommands:
             # it's a subcommand, and *not* a flag or class parameter
             app = Application()
             app.initialize_subcommand(subc, subargv)
@@ -60,24 +58,24 @@ def _preparse_for_stopping_flags(Application, argv):
     # version), we want to only search the arguments up to the first
     # occurrence of '--', which we're calling interpreted_argv.
     try:
-        interpreted_argv = argv[:argv.index('--')]
+        interpreted_argv = argv[: argv.index("--")]
     except ValueError:
         interpreted_argv = argv
 
     # Catch any help calls.
-    if any(x in interpreted_argv for x in ('-h', '--help-all', '--help')):
+    if any(x in interpreted_argv for x in ("-h", "--help-all", "--help")):
         app = Application()
-        app.print_help('--help-all' in interpreted_argv)
+        app.print_help("--help-all" in interpreted_argv)
         app.exit(0)
 
     # Catch version commands
-    if '--version' in interpreted_argv or '-V' in interpreted_argv:
+    if "--version" in interpreted_argv or "-V" in interpreted_argv:
         app = Application()
         app.print_version()
         app.exit(0)
 
     # Catch generate-config commands.
-    if '--generate-config' in interpreted_argv:
+    if "--generate-config" in interpreted_argv:
         app = Application()
         app.write_default_config()
         app.exit(0)
@@ -87,8 +85,10 @@ class ExtensionAppJinjaMixin(HasTraits):
     """Use Jinja templates for HTML templates on top of an ExtensionApp."""
 
     jinja2_options = Dict(
-        help=_("""Options to pass to the jinja2 environment for this
-        """)
+        help=_i18n(
+            """Options to pass to the jinja2 environment for this
+        """
+        )
     ).tag(config=True)
 
     def _prepare_templates(self):
@@ -96,25 +96,19 @@ class ExtensionAppJinjaMixin(HasTraits):
         self.initialize_templates()
         # Add templates to web app settings if extension has templates.
         if len(self.template_paths) > 0:
-            self.settings.update({
-                "{}_template_paths".format(self.name): self.template_paths
-            })
+            self.settings.update({"{}_template_paths".format(self.name): self.template_paths})
 
         # Create a jinja environment for logging html templates.
         self.jinja2_env = Environment(
             loader=FileSystemLoader(self.template_paths),
-            extensions=['jinja2.ext.i18n'],
+            extensions=["jinja2.ext.i18n"],
             autoescape=True,
             **self.jinja2_options
         )
 
-
         # Add the jinja2 environment for this extension to the tornado settings.
-        self.settings.update(
-            {
-                "{}_jinja2_env".format(self.name): self.jinja2_env
-            }
-        )
+        self.settings.update({"{}_jinja2_env".format(self.name): self.jinja2_env})
+
 
 # -----------------------------------------------------------------------------
 # ExtensionApp
@@ -124,6 +118,7 @@ class ExtensionAppJinjaMixin(HasTraits):
 class JupyterServerExtensionException(Exception):
     """Exception class for raising for Server extensions errors."""
 
+
 # -----------------------------------------------------------------------------
 # ExtensionApp
 # -----------------------------------------------------------------------------
@@ -140,6 +135,7 @@ class ExtensionApp(JupyterApp):
         class method. This method can be set as a entry_point in
         the extensions setup.py
     """
+
     # Subclasses should override this trait. Tells the server if
     # this extension allows other other extensions to be loaded
     # side-by-side when launched directly.
@@ -154,7 +150,6 @@ class ExtensionApp(JupyterApp):
     # the default value to False if they don't offer a browser
     # based frontend.
     open_browser = Bool(
-        True,
         help="""Whether to open in a browser after starting.
         The specific browser used is platform dependent and
         determined by the python standard library `webbrowser`
@@ -163,6 +158,10 @@ class ExtensionApp(JupyterApp):
         """
     ).tag(config=True)
 
+    @default("open_browser")
+    def _default_open_browser(self):
+        return self.serverapp.config["ServerApp"].get("open_browser", True)
+
     # The extension name used to name the jupyter config
     # file, jupyter_{name}_config.
     # This should also match the jupyter subcommand used to launch
@@ -171,7 +170,11 @@ class ExtensionApp(JupyterApp):
 
     @classmethod
     def get_extension_package(cls):
-        return cls.__module__.split('.')[0]
+        parts = cls.__module__.split(".")
+        if is_namespace_package(parts[0]):
+            # in this case the package name is `<namespace>.<package>`.
+            return ".".join(parts[0:2])
+        return parts[0]
 
     @classmethod
     def get_extension_point(cls):
@@ -180,6 +183,17 @@ class ExtensionApp(JupyterApp):
     # Extension URL sets the default landing page for this extension.
     extension_url = "/"
 
+    default_url = Unicode().tag(config=True)
+
+    @default("default_url")
+    def _default_url(self):
+        return self.extension_url
+
+    file_url_prefix = Unicode("notebooks")
+
+    # Is this linked to a serverapp yet?
+    _linked = Bool(False)
+
     # Extension can configure the ServerApp from the command-line
     classes = [
         ServerApp,
@@ -190,14 +204,11 @@ class ExtensionApp(JupyterApp):
 
     _log_formatter_cls = LogFormatter
 
-    # Whether this app is the starter app
-    _is_starter_app = False
-
-    @default('log_level')
+    @default("log_level")
     def _default_log_level(self):
         return logging.INFO
 
-    @default('log_format')
+    @default("log_format")
     def _default_log_format(self):
         """override default log format to include date & time"""
         return u"%(color)s[%(levelname)1.1s %(asctime)s.%(msecs).03d %(name)s]%(end_color)s %(message)s"
@@ -206,40 +217,38 @@ class ExtensionApp(JupyterApp):
         help="""Url where the static assets for the extension are served."""
     ).tag(config=True)
 
-    @default('static_url_prefix')
+    @default("static_url_prefix")
     def _default_static_url_prefix(self):
-        static_url = "static/{name}/".format(
-            name=self.name
-        )
+        static_url = "static/{name}/".format(name=self.name)
         return url_path_join(self.serverapp.base_url, static_url)
 
-    static_paths = List(Unicode(),
+    static_paths = List(
+        Unicode(),
         help="""paths to search for serving static files.
 
         This allows adding javascript/css to be available from the notebook server machine,
         or overriding individual files in the IPython
-        """
+        """,
     ).tag(config=True)
 
-    template_paths = List(Unicode(),
-        help=_("""Paths to search for serving jinja templates.
+    template_paths = List(
+        Unicode(),
+        help=_i18n(
+            """Paths to search for serving jinja templates.
 
-        Can be used to override templates from notebook.templates.""")
+        Can be used to override templates from notebook.templates."""
+        ),
     ).tag(config=True)
 
-    settings = Dict(
-        help=_("""Settings that will passed to the server.""")
-    ).tag(config=True)
+    settings = Dict(help=_i18n("""Settings that will passed to the server.""")).tag(config=True)
 
-    handlers = List(
-        help=_("""Handlers appended to the server.""")
-    ).tag(config=True)
+    handlers = List(help=_i18n("""Handlers appended to the server.""")).tag(config=True)
 
     def _config_file_name_default(self):
         """The default config file name."""
         if not self.name:
-            return ''
-        return 'jupyter_{}_config'.format(self.name.replace('-','_'))
+            return ""
+        return "jupyter_{}_config".format(self.name.replace("-", "_"))
 
     def initialize_settings(self):
         """Override this method to add handling of settings."""
@@ -259,7 +268,7 @@ class ExtensionApp(JupyterApp):
         """
         traits = self.class_own_traits().keys()
         self.extension_config = Config({t: getattr(self, t) for t in traits})
-        self.settings['{}_config'.format(self.name)] = self.extension_config
+        self.settings["{}_config".format(self.name)] = self.extension_config
 
     def _prepare_settings(self):
         # Make webapp settings accessible to initialize_settings method
@@ -267,10 +276,12 @@ class ExtensionApp(JupyterApp):
         self.settings.update(**webapp.settings)
 
         # Add static and template paths to settings.
-        self.settings.update({
-            "{}_static_paths".format(self.name): self.static_paths,
-            "{}".format(self.name): self,
-        })
+        self.settings.update(
+            {
+                "{}_static_paths".format(self.name): self.static_paths,
+                "{}".format(self.name): self,
+            }
+        )
 
         # Get setting defined by subclass using initialize_settings method.
         self.initialize_settings()
@@ -288,13 +299,13 @@ class ExtensionApp(JupyterApp):
         new_handlers = []
         for handler_items in self.handlers:
             # Build url pattern including base_url
-            pattern = url_path_join(webapp.settings['base_url'], handler_items[0])
+            pattern = url_path_join(webapp.settings["base_url"], handler_items[0])
             handler = handler_items[1]
 
             # Get handler kwargs, if given
             kwargs = {}
             if issubclass(handler, ExtensionHandlerMixin):
-                kwargs['name'] = self.name
+                kwargs["name"] = self.name
 
             try:
                 kwargs.update(handler_items[2])
@@ -312,30 +323,28 @@ class ExtensionApp(JupyterApp):
             # Construct handler.
             handler = (
                 static_url,
-                webapp.settings['static_handler_class'],
-                {'path': self.static_paths}
+                webapp.settings["static_handler_class"],
+                {"path": self.static_paths},
             )
             new_handlers.append(handler)
 
-        webapp.add_handlers('.*$', new_handlers)
+        webapp.add_handlers(".*$", new_handlers)
 
     def _prepare_templates(self):
         # Add templates to web app settings if extension has templates.
         if len(self.template_paths) > 0:
-            self.settings.update({
-                "{}_template_paths".format(self.name): self.template_paths
-            })
+            self.settings.update({"{}_template_paths".format(self.name): self.template_paths})
         self.initialize_templates()
 
-    @classmethod
-    def _jupyter_server_config(cls):
+    def _jupyter_server_config(self):
         base_config = {
             "ServerApp": {
-                "jpserver_extensions": {cls.get_extension_package(): True},
-                "default_url": cls.extension_url
+                "default_url": self.default_url,
+                "open_browser": self.open_browser,
+                "file_url_prefix": self.file_url_prefix,
             }
         }
-        base_config["ServerApp"].update(cls.serverapp_config)
+        base_config["ServerApp"].update(self.serverapp_config)
         return base_config
 
     def _link_jupyter_server_extension(self, serverapp):
@@ -346,6 +355,10 @@ class ExtensionApp(JupyterApp):
         the command line contains traits for the ExtensionApp
         or the ExtensionApp's config files have server
         settings.
+
+        Note, the ServerApp has not initialized the Tornado
+        Web Application yet, so do not try to affect the
+        `web_app` attribute.
         """
         self.serverapp = serverapp
         # Load config from an ExtensionApp's config files.
@@ -365,23 +378,8 @@ class ExtensionApp(JupyterApp):
         # ServerApp, do it here.
         # i.e. ServerApp traits <--- ExtensionApp config
         self.serverapp.update_config(self.config)
-
-    @classmethod
-    def initialize_server(cls, argv=[], load_other_extensions=True, **kwargs):
-        """Creates an instance of ServerApp where this extension is enabled
-        (superceding disabling found in other config from files).
-
-        This is necessary when launching the ExtensionApp directly from
-        the `launch_instance` classmethod.
-        """
-        # The ExtensionApp needs to add itself as enabled extension
-        # to the jpserver_extensions trait, so that the ServerApp
-        # initializes it.
-        config = Config(cls._jupyter_server_config())
-        serverapp = ServerApp.instance(**kwargs, argv=[], config=config)
-        cls._is_starter_app = True
-        serverapp.initialize(argv=argv, find_extensions=load_other_extensions)
-        return serverapp
+        # Acknowledge that this extension has been linked.
+        self._linked = True
 
     def initialize(self):
         """Initialize the extension app. The
@@ -416,9 +414,11 @@ class ExtensionApp(JupyterApp):
         # Start the server.
         self.serverapp.start()
 
+    async def stop_extension(self):
+        """Cleanup any resources managed by this extension."""
+
     def stop(self):
-        """Stop the underlying Jupyter server.
-        """
+        """Stop the underlying Jupyter server."""
         self.serverapp.stop()
         self.serverapp.clear_instance()
 
@@ -435,41 +435,112 @@ class ExtensionApp(JupyterApp):
         except KeyError:
             extension = cls()
             extension._link_jupyter_server_extension(serverapp)
-        if cls._is_starter_app:
-            serverapp._starter_app = extension
         extension.initialize()
         return extension
 
     @classmethod
     def load_classic_server_extension(cls, serverapp):
-        """Enables extension to be loaded as classic Notebook (jupyter/notebook) extension.
-        """
+        """Enables extension to be loaded as classic Notebook (jupyter/notebook) extension."""
         extension = cls()
         extension.serverapp = serverapp
         extension.load_config_file()
         extension.update_config(serverapp.config)
         extension.parse_command_line(serverapp.extra_args)
         # Add redirects to get favicons from old locations in the classic notebook server
-        extension.handlers.extend([
-            (r"/static/favicons/favicon.ico", RedirectHandler,
-                {"url": url_path_join(serverapp.base_url, "static/base/images/favicon.ico")}),
-            (r"/static/favicons/favicon-busy-1.ico", RedirectHandler,
-                {"url": url_path_join(serverapp.base_url, "static/base/images/favicon-busy-1.ico")}),
-            (r"/static/favicons/favicon-busy-2.ico", RedirectHandler,
-                {"url": url_path_join(serverapp.base_url, "static/base/images/favicon-busy-2.ico")}),
-            (r"/static/favicons/favicon-busy-3.ico", RedirectHandler,
-                {"url": url_path_join(serverapp.base_url, "static/base/images/favicon-busy-3.ico")}),
-            (r"/static/favicons/favicon-file.ico", RedirectHandler,
-                {"url": url_path_join(serverapp.base_url, "static/base/images/favicon-file.ico")}),
-            (r"/static/favicons/favicon-notebook.ico", RedirectHandler,
-                {"url": url_path_join(serverapp.base_url, "static/base/images/favicon-notebook.ico")}),
-            (r"/static/favicons/favicon-terminal.ico", RedirectHandler,
-                {"url": url_path_join(serverapp.base_url, "static/base/images/favicon-terminal.ico")}),
-            (r"/static/logo/logo.png", RedirectHandler,
-                {"url": url_path_join(serverapp.base_url, "static/base/images/logo.png")}),
-        ])
+        extension.handlers.extend(
+            [
+                (
+                    r"/static/favicons/favicon.ico",
+                    RedirectHandler,
+                    {"url": url_path_join(serverapp.base_url, "static/base/images/favicon.ico")},
+                ),
+                (
+                    r"/static/favicons/favicon-busy-1.ico",
+                    RedirectHandler,
+                    {
+                        "url": url_path_join(
+                            serverapp.base_url, "static/base/images/favicon-busy-1.ico"
+                        )
+                    },
+                ),
+                (
+                    r"/static/favicons/favicon-busy-2.ico",
+                    RedirectHandler,
+                    {
+                        "url": url_path_join(
+                            serverapp.base_url, "static/base/images/favicon-busy-2.ico"
+                        )
+                    },
+                ),
+                (
+                    r"/static/favicons/favicon-busy-3.ico",
+                    RedirectHandler,
+                    {
+                        "url": url_path_join(
+                            serverapp.base_url, "static/base/images/favicon-busy-3.ico"
+                        )
+                    },
+                ),
+                (
+                    r"/static/favicons/favicon-file.ico",
+                    RedirectHandler,
+                    {
+                        "url": url_path_join(
+                            serverapp.base_url, "static/base/images/favicon-file.ico"
+                        )
+                    },
+                ),
+                (
+                    r"/static/favicons/favicon-notebook.ico",
+                    RedirectHandler,
+                    {
+                        "url": url_path_join(
+                            serverapp.base_url, "static/base/images/favicon-notebook.ico"
+                        )
+                    },
+                ),
+                (
+                    r"/static/favicons/favicon-terminal.ico",
+                    RedirectHandler,
+                    {
+                        "url": url_path_join(
+                            serverapp.base_url, "static/base/images/favicon-terminal.ico"
+                        )
+                    },
+                ),
+                (
+                    r"/static/logo/logo.png",
+                    RedirectHandler,
+                    {"url": url_path_join(serverapp.base_url, "static/base/images/logo.png")},
+                ),
+            ]
+        )
         extension.initialize()
 
+    @classmethod
+    def initialize_server(cls, argv=[], load_other_extensions=True, **kwargs):
+        """Creates an instance of ServerApp and explicitly sets
+        this extension to enabled=True (i.e. superceding disabling
+        found in other config from files).
+
+        The `launch_instance` method uses this method to initialize
+        and start a server.
+        """
+        jpserver_extensions = {cls.get_extension_package(): True}
+        find_extensions = cls.load_other_extensions
+        if "jpserver_extensions" in cls.serverapp_config:
+            jpserver_extensions.update(cls.serverapp_config["jpserver_extensions"])
+            cls.serverapp_config["jpserver_extensions"] = jpserver_extensions
+            find_extensions = False
+        serverapp = ServerApp.instance(jpserver_extensions=jpserver_extensions, **kwargs)
+        serverapp.aliases.update(cls.aliases)
+        serverapp.initialize(
+            argv=argv,
+            starter_extension=cls.name,
+            find_extensions=find_extensions,
+        )
+        return serverapp
+
     @classmethod
     def launch_instance(cls, argv=None, **kwargs):
         """Launch the extension like an application. Initializes+configs a stock server
@@ -481,27 +552,28 @@ class ExtensionApp(JupyterApp):
             args = sys.argv[1:]  # slice out extension config.
         else:
             args = argv
-        # Check for subcommands
+
+        # Handle all "stops" that could happen before
+        # continuing to launch a server+extension.
         subapp = _preparse_for_subcommand(cls, args)
         if subapp:
             subapp.start()
-        else:
-            # Check for help, version, and generate-config arguments
-            # before initializing server to make sure these
-            # arguments trigger actions from the extension not the server.
-            _preparse_for_stopping_flags(cls, args)
-            # Get a jupyter server instance.
-            serverapp = cls.initialize_server(
-                argv=args,
-                load_other_extensions=cls.load_other_extensions
+            return
+
+        # Check for help, version, and generate-config arguments
+        # before initializing server to make sure these
+        # arguments trigger actions from the extension not the server.
+        _preparse_for_stopping_flags(cls, args)
+        serverapp = cls.initialize_server(argv=args)
+
+        # Log if extension is blocking other extensions from loading.
+        if not cls.load_other_extensions:
+            serverapp.log.info(
+                "{ext_name} is running without loading "
+                "other extensions.".format(ext_name=cls.name)
             )
-            # Log if extension is blocking other extensions from loading.
-            if not cls.load_other_extensions:
-                serverapp.log.info(
-                    "{ext_name} is running without loading "
-                    "other extensions.".format(ext_name=cls.name)
-                )
-            try:
-                serverapp.start()
-            except NoStart:
-                pass
+        # Start the server.
+        try:
+            serverapp.start()
+        except NoStart:
+            pass
